#ifndef _RPC_CONFIG_H_
#define _RPC_CONFIG_H_

#include <string>
#include <vector>
#include <unordered_map>
#include "json/Json.h"

namespace srpc
{

class ClientRPCConfig
{
public:
    using ErrorPageMap = std::unordered_map<int, std::string>;

    bool load(const char *file);

    unsigned short server_port() const { return this->s_port; }
    const char *server_cert_file() const { return this->s_cert_file.c_str(); }
    const char *server_file_key() const { return this->s_file_key.c_str(); }
    unsigned short client_port() const { return this->c_port; }
    const char *client_host() const { return this->c_host.c_str(); }
    int redirect_max() const { return this->c_redirect_max; }
    int retry_max() const { return this->c_retry_max; }
    const char *client_user_name() const { return this->c_user_name.c_str(); }
    const char *client_password() const { return this->c_password.c_str(); }
    const char *get_root_path() const { return this->root_path.c_str(); }
    const ErrorPageMap& get_error_page() const { return this->error_page; }

    const char* get_conversion_instance() { return conversion_instance.c_str(); }
    const char* get_conversion_type() { return conversion_type.c_str(); }
    const char* get_conversion_resource_path() { return conversion_resource_path.c_str(); }
    const char* get_output_path() { return output_path.c_str(); }
    const char* get_model_path() { return model_path.c_str(); }
    const char* get_pdf_root_path() { return pdf_root_path.c_str(); }
    const char* get_time_stamp() { return time_stamp.c_str(); }

    void add_http_put_count() { http_put_count++; }
    void add_http_get_count() { http_get_count++; }
    int get_http_put_count() { return http_put_count; }
    int get_http_get_count() { return http_get_count; }
    int get_outer_batch_size() { return outer_batch_size; }
    int get_inner_batch_size() { return inner_batch_size; }

public:
    ClientRPCConfig() : s_port(0), c_port(0), c_redirect_max(0), c_retry_max(0) { }
    ~ClientRPCConfig();

private:
    void load_server();
    void load_client();
    void load_conversion();

    wfrest::Json data;
    unsigned short s_port;
    std::string s_cert_file;
    std::string s_file_key;
    std::string c_host;
    unsigned short c_port;
    int c_redirect_max;
    int c_retry_max;
    std::string c_user_name;
    std::string c_password;
    std::string root_path;
    ErrorPageMap error_page;

    int http_get_count = 0;
    int http_put_count = 0;
    int outer_batch_size;
    int inner_batch_size;
    std::string pdf_root_path;
    std::string conversion_resource_path;
    std::string conversion_instance;
    std::string conversion_type;
    std::string output_path;
    std::string model_path;
    std::string time_stamp;
};

}

#endif
